defmodule ClickHouse.DataType do
  @moduledoc """
  Functions for working with ClickHouse data types.
  """

  @doc """
  Encodes the provided data using the `ClickHouse.DataType.Encodable` protocol.
  """
  @spec encode(any()) :: iodata()
  defdelegate encode(data), to: ClickHouse.DataType.Encodable

  @spec intersperse_map(list, any, any, any) :: any
  def intersperse_map(list, seperator, mapper, acc \\ [])

  def intersperse_map([], _separator, _mapper, acc), do: acc

  def intersperse_map([elem], _separator, mapper, acc) do
    [acc | mapper.(elem)]
  end

  def intersperse_map([elem | rest], separator, mapper, acc) do
    intersperse_map(rest, separator, mapper, [acc, mapper.(elem), separator])
  end
end
