defmodule ClickHouse.Format do
  @moduledoc """
  A behaviour to implement formatting of ClickHouse queries and results.
  """

  @typedoc """
  Input and output formats from ClickHouse
  """
  @type t :: module()

  @typedoc """
  A ClickHouse format name.
  """
  @type name :: binary()

  @doc """
  A callback to return the names used for the format.
  """
  @callback names :: [name()]

  @doc """
  A callback to decode results from the format.
  """
  @callback decode(result :: ClickHouse.Result.t()) :: ClickHouse.Result.t()

  @doc """
  A callback to encode queries with the format.
  """
  @callback encode(query :: ClickHouse.Query.t()) :: ClickHouse.Query.t()
end
