defmodule ClickHouse.Format.Values do
  @moduledoc """
  An implementation of the ClickHouse `"Values"` format.
  """

  @behaviour ClickHouse.Format

  alias ClickHouse.{DataType, Query}

  ################################
  # ClickHouse.Format Callbacks
  ################################

  @impl ClickHouse.Format
  @spec names() :: [binary()]
  def names, do: ["Values"]

  @impl ClickHouse.Format
  @spec decode(ClickHouse.Result.t()) :: no_return()
  def decode(_result) do
    raise RuntimeError, "Values decode not implemented."
  end

  @impl ClickHouse.Format
  @spec encode(ClickHouse.Query.t()) :: ClickHouse.Query.t()
  def encode(%Query{command: :insert, params_count: count}) when count > 1 do
    raise ArgumentError, "Values insert statement can only support a single parameter."
  end

  def encode(%Query{command: :insert, params_count: 0, params: [_ | _]}) do
    raise ArgumentError, "Values insert statement with params must include a '$1'."
  end

  def encode(%Query{command: :insert, params_count: 1, params: params} = query) do
    params = DataType.intersperse_map(params, ",", &encode_row/1)
    statement = String.trim_trailing(query.statement, "$1")
    Query.put_encoded(query, [statement, params])
  end

  ################################
  # Private API
  ################################

  defp encode_row(row) do
    ["(", DataType.intersperse_map(row, ",", &DataType.encode/1), ")"]
  end
end
